/*
 *  Copyright (C) 2010-2012 Agrosense info@agrosense.eu
 * 
 *  Licensed under the Eclipse Public License - v 1.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 * 
 *           http://www.eclipse.org/legal/epl-v10.html
 * 
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */
package nl.cloudfarming.client.field.shape;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.Collections;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;
import nl.cloudfarming.client.field.model.Field;
import nl.cloudfarming.client.field.model.FieldManager;
import nl.cloudfarming.client.field.model.Shape;
import org.jdesktop.swingworker.SwingWorker;
import org.netbeans.api.progress.ProgressHandle;
import org.netbeans.api.progress.ProgressHandleFactory;
import org.netbeans.api.project.FileOwnerQuery;
import org.netbeans.api.project.Project;
import org.openide.awt.ActionID;
import org.openide.awt.ActionReference;
import org.openide.awt.ActionReferences;
import org.openide.awt.ActionRegistration;
import org.openide.util.Lookup;
import org.openide.util.NbBundle;

/**
 *
 * @author Timon Veenstra <monezz@gmail.com>
 */
@ActionID(category = "Task", id = "nl.cloudfarming.client.field.shape.CreateFieldsFromShapeAction")
@ActionRegistration(iconBase = "nl/cloudfarming/client/field/model/shape_icon_polygon.png", displayName = "#CTL_CreateFieldsFromShape")
@ActionReferences(value = {
    @ActionReference(path = "Loaders/application/x-esri-shape/Actions", position = 3000)})
@NbBundle.Messages({"CTL_CreateFieldsFromShape=Convert to field", "CreateFieldsFromShapeAction_progress_handler_msg=Converting shapes to fields"})
public class CreateFieldsFromShapeAction implements ActionListener {

    private final List<FieldImportCookie> cookies;
    private static final Logger LOGGER = Logger.getLogger(CreateFieldsFromShapeAction.class.getName());

    public CreateFieldsFromShapeAction(List<FieldImportCookie> cookies) {
        this.cookies = cookies;
    }
    
    
    @Override
    public void actionPerformed(ActionEvent e) {
        LOGGER.log(Level.FINEST, "performing convert to task action on {0} objects", cookies.size());
        for (FieldImportCookie cookie : cookies) {
            ShapeDataObject shapeDataObject = cookie.getShapeDataObject();
            final List<Shape> shapes = shapeDataObject.getShapes();
            final String dataObjectFileName = shapeDataObject.getPrimaryFile().getName();
            final Project project = FileOwnerQuery.getOwner(shapeDataObject.getPrimaryFile());
            final FieldManager fieldManager = Lookup.getDefault().lookup(FieldManager.class);

            //FIXME create multi process handler
            ProgressHandle progressHandle = ProgressHandleFactory.createHandle(Bundle.CreateFieldsFromShapeAction_progress_handler_msg());
            //start the progressHandle and set its limit to the number of shapes
            //contained by the shapeDataObject
            progressHandle.start(shapes.size());
            SwingWorker worker;
            final PropertyChangeListener progressListener = new ProgressListener(progressHandle);


                    worker = new FieldWorker(fieldManager, shapes, project, progressListener);
                    LOGGER.finest("executing field worker..");
            worker.execute();
        }
    }
    
    //TODO TV: inner classes copied from CreateTaskFromShapeAction in task-shape module
   /**
     * Abstract class that serves as a superclass for FieldWorker and
     * TreatmentZoneWorker. This class contains the fields needed by the Field-
     * and TreatmentZoneWorkers.
     */
    private abstract class ShapeWorker extends SwingWorker<List<Field>, Object> {

        protected List<Shape> shapes;
        protected FieldManager fieldManager;
        protected Project project;
        protected PropertyChangeListener progressListener;

        public ShapeWorker(FieldManager fieldManager, List<Shape> shapes, Project project, PropertyChangeListener progressListener) {
            this.fieldManager = fieldManager;
            this.shapes = shapes;
            this.project = project;
            this.progressListener = progressListener;
        }

        @Override
        protected List<Field> doInBackground() throws Exception {
            return Collections.EMPTY_LIST;
        }
    }

    /**
     * This class is a specific implementation of the ShapeWorker class which is
     * used to import fields into a project in the background.
     */
    private class FieldWorker extends ShapeWorker {

        public FieldWorker(FieldManager fieldManager, List<Shape> shapes, Project project, PropertyChangeListener progressListener) {
            super(fieldManager, shapes, project, progressListener);
        }

        @Override
        protected List<Field> doInBackground() throws Exception {
            return fieldManager.importFields(shapes, project, progressListener);
        }
    }

    
    /**
     * Small inner class responsible for updating the progress of the
     * progressHandle and calling finish on the progressHandle when it is
     * finished.
     */
    private class ProgressListener implements PropertyChangeListener {

        private final ProgressHandle handle;

        public ProgressListener(ProgressHandle handle) {
            this.handle = handle;
        }        
        
        @Override
        public void propertyChange(PropertyChangeEvent evt) {
            if (FieldManager.PROP_PROGRESS.equals(evt.getPropertyName())) {
                LOGGER.log(Level.FINEST, "recieved notification of progress to {0}", (Integer) evt.getNewValue());
                handle.progress((Integer) evt.getNewValue());
            } else if (FieldManager.PROP_FINISH.equals(evt.getPropertyName())) {
                handle.finish();
            }
        }
    }    
}
