/*
 * Copyright (c) 2012 AgroSense.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    AgroSense - initial API and implementation and/or initial documentation
 */
package nl.cloudfarming.client.field.shape;

import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.util.Collection;
import java.util.Collections;
import java.util.LinkedList;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;
import org.openide.filesystems.FileObject;
import org.openide.filesystems.FileUtil;

/**
 *
 * @author johan
 */
public class ShapePropertyMappingRegistry {
    public static final String PROP_MAPPINGS = "mappings";
    
    public static final String FARMWORKS_NAME_PROPERTY = "ID";
    public static final String FARMWORKS_VALUE_PROPERTY = "RATE_CAL";

    private static final String REGISTRY_FILE = "shapePropertyMappings.ser";
    private static final Logger LOGGER = Logger.getLogger(ShapePropertyMappingRegistry.class.getName());
    
    private static ShapePropertyMappingRegistry instance;
    
    private List<ShapePropertyMapping> mappings;
    private transient PropertyChangeSupport propertyChangeSupport;
    
    private ShapePropertyMappingRegistry() {
        load();
    }
    
    public static synchronized ShapePropertyMappingRegistry getInstance() {
        if (instance == null) {
            instance = new ShapePropertyMappingRegistry();
        }
        return instance;
    }
    
    private void load() {
        FileObject fo = getRegistryFile(false);
        if (fo == null) {
            // initialize: 
            mappings = new LinkedList<>();
            mappings.add(new ShapePropertyMapping(FARMWORKS_VALUE_PROPERTY));
            save();
        } else {
            // read existing file:
            try (ObjectInputStream ois = new ObjectInputStream(fo.getInputStream())) {
                mappings = (List<ShapePropertyMapping>) ois.readObject();
            } catch (IOException | ClassNotFoundException ex) {
                LOGGER.log(Level.SEVERE, "could not load registry from {0} with error: {1}", new Object[]{fo.getPath(), ex.getMessage()});
            }
        }
    }
    
    private synchronized void save() {
        FileObject fo = getRegistryFile(true);
        try (ObjectOutputStream oos = new ObjectOutputStream(fo.getOutputStream())) {
            oos.writeObject(mappings);
        } catch (IOException ex) {
            LOGGER.log(Level.SEVERE, "could not save registry to {0} with error: {1}", new Object[]{fo.getPath(), ex.getMessage()});
        }
    }

    private FileObject getRegistryFile(boolean create) {
        FileObject fo = getRegistryFolder().getFileObject(REGISTRY_FILE);
        if (fo == null && create) {
            try {
                fo = getRegistryFolder().createData(REGISTRY_FILE);
            } catch (IOException ex) {
                LOGGER.log(Level.SEVERE, "could not create registry file with error: {1}", new Object[]{ex.getMessage()});
            }
        }
        return fo;
    }
    
    FileObject getRegistryFolder() {
        return FileUtil.getConfigRoot();
    }
    
    public void add(ShapePropertyMapping mapping) {
        if (!mappings.contains(mapping)) {
            mappings.add(mapping);
            save();
            getPropertyChangeSupport().firePropertyChange(PROP_MAPPINGS, null, null);
        }
    }
    
    public void remove(ShapePropertyMapping mapping) {
        if (mappings.contains(mapping)) {
            mappings.remove(mapping);
            save();
            getPropertyChangeSupport().firePropertyChange(PROP_MAPPINGS, null, null);
        }
    }
    
    public void removeAll(Collection<ShapePropertyMapping> mappingsToRemove) {
        if (mappings.removeAll(mappingsToRemove)) {
            save();
            getPropertyChangeSupport().firePropertyChange(PROP_MAPPINGS, null, null);
        }
    }
    
    public List<ShapePropertyMapping> findAll() {
        return Collections.unmodifiableList(mappings);
    }
    
    /**
     * 
     * @param nameAttributes
     * @param valueAttributes
     * @return the first registered mapping that matches both a name and a value attribute.
     */
    public ShapePropertyMapping findMatch(List<String> nameAttributes, List<String> valueAttributes) {
        return findMatch(mappings, nameAttributes, valueAttributes);
    }
        
    /**
     * 
     * @param nameAttributes
     * @param valueAttributes
     * @return all registered mappings that match both a name and a value attribute.
     */
    public List<ShapePropertyMapping> findMatches(List<String> nameAttributes, List<String> valueAttributes) {
        return findMatches(mappings, nameAttributes, valueAttributes);
    }
    
    protected PropertyChangeSupport getPropertyChangeSupport() {
        if (propertyChangeSupport == null) {
            propertyChangeSupport = new PropertyChangeSupport(this);
        }
        return propertyChangeSupport;
    }

    public void addPropertyChangeListener(PropertyChangeListener listener) {
        assert listener != null;
        getPropertyChangeSupport().addPropertyChangeListener(listener);
    }

    public void addPropertyChangeListener(String property, PropertyChangeListener listener) {
        assert listener != null;
        getPropertyChangeSupport().addPropertyChangeListener(property, listener);
    }

    public void removePropertyChangeListener(PropertyChangeListener listener) {
        getPropertyChangeSupport().removePropertyChangeListener(listener);
    }

    public void removePropertyChangeListener(String property, PropertyChangeListener listener) {
        getPropertyChangeSupport().removePropertyChangeListener(property, listener);
    }
    
    /**
     * 
     * @param mappings 
     * @param nameAttributes
     * @param valueAttributes
     * @return the first input mapping that matches both a name and a value attribute.
     */
    public static ShapePropertyMapping findMatch(List<ShapePropertyMapping> mappings, List<String> nameAttributes, List<String> valueAttributes) {
        for (ShapePropertyMapping mapping : mappings) {
            if (valueAttributes.contains(mapping.getValueProperty())) {
                return mapping;
            }
        }
        return null;
    }
    
    /**
     * 
     * @param mappings 
     * @param nameAttributes
     * @param valueAttributes
     * @return all input mappings that match both a name and a value attribute.
     */
    public static List<ShapePropertyMapping> findMatches(List<ShapePropertyMapping> mappings, List<String> nameAttributes, List<String> valueAttributes) {
        List<ShapePropertyMapping> matches = new LinkedList<>();
        for (ShapePropertyMapping mapping : mappings) {
            if (valueAttributes.contains(mapping.getValueProperty())) {
                matches.add(mapping);
            }
        }
        return matches;
    }
        
}
